//
// Patron pour la generation d'une classe derivee dans ROOT
// FFasterDataReader derive de FBlockDataReader
//

#ifndef FFasterDataReader_h
#define FFasterDataReader_h

#include "FBlockDataReader.h"
#include "FParamBrut.h"

// Pour la gestion des trames Faster
const Int_t headerDataLoadSize = 12; // 12 octets d'entete de trame data
const Int_t maxLoadDataTrame = 704;  // Nombre max de data de type oscillo dans une trame:
// Normalement ETHERNET_MTU-UDP vaut 1500, mais la limite est fixee ici par le GEDEK a 1456 !!
// (GEDEK-UDP_header-Faster_Header_Trame-headerDataLoadSize-samplingMetaDataHeader)/2 = (1456-8-12-28)/2 = 704
const Int_t samplingMetaDataHeader = 28; // 28 octets en entete des donnees Sampling 
// En entete: 7 mots x 32bits = 7 mots x 4 x 8bits = 28 octets

// Champ de bits pour donnee de type QDC
typedef struct qdc_data{
  signed   value    : 31; // qdc value
  unsigned satured  :  1; // satured bit
} qdc_data;


// Champ de bits pour donnee de type ADC
typedef struct adc_data{
  signed   value    : 22; // qdc value
  unsigned pad	    :  8; // [22-29] not used
  unsigned satured  :  1; // satured bit
  unsigned pileup   :  1; // pileup bit
} adc_data;

// Champ de bits pour donnee de type Periode du module RF
typedef struct period_data{
  unsigned value    : 31; // period value
  unsigned satured  :  1; // satured bit
} period_data;

// Definition des types de donnees FASTER interessantes pour RHB:
//
// System [0-9]
//const Int_t SYST = 1;
//----------------------
// Group [10]
const UShort_t GROUP = 10;
//----------------------
// Time reference types [11-18]
const UShort_t TREF = 11;     // Special type without load (Timestamp in Data Header only)
const UShort_t TREF_TDC = 12; // Special type with a Time Precision measure in pico seconds in load (and Timestamp in Data Header too)
//----------------------
// RF module [19-20]
const UShort_t RF = 19;
// RF counters [20]
const UShort_t RF_COUNT = 20;
//----------------------
// Oscillograms [21]
const UShort_t OSC = 21;
//----------------------
// QDC System types [41-44]
//----------------------
const UShort_t QDCS = 40; // Internal family number to compute the label number for all QDC data in RHB
//----------------------
const UShort_t QDC1 = 41;
const UShort_t QDC2 = 42;
const UShort_t QDC3 = 43;
const UShort_t QDC4 = 44;
//----------------------
// QDC User [45-49]
// add them here ...
//----------------------
// QDC counters [50-60]
const UShort_t QCOUNT = 50;
//----------------------
// ADC System [61]
const UShort_t ADC1 = 61;
//----------------------
// ADC User [62-69]
//  ... add them here ...
//----------------------
// ADC counters [70-80]
const UShort_t ACOUNT = 70;
//----------------------
// QDC_TDC types [141-144]
//----------------------
const UShort_t QDC_TDCS = 140; // Internal family number to compute the label number for all QDC_TDC data in RHB
//----------------------
const UShort_t QDC_TDC1 = 141;
const UShort_t QDC_TDC2 = 142;
const UShort_t QDC_TDC3 = 143;
const UShort_t QDC_TDC4 = 144;
//----------------------
// QDC_TOF types [241-244]
//----------------------
const UShort_t QDC_TOFS = 240; // Internal family number to compute the label number for all QDC_TOF data in RHB
//----------------------
const UShort_t QDC_TOF1 = 241;
const UShort_t QDC_TOF2 = 242;
const UShort_t QDC_TOF3 = 243;
const UShort_t QDC_TOF4 = 244;
// Fin definition des types de donnees FASTER


class FFasterDataReader : public FBlockDataReader {

  protected:
    FDevice *FasterDevice;
    UShort_t type;
    UShort_t label;
    ULong64_t clock;
    UShort_t load;
    TString fileParamDesc;
    Int_t cptParam;
    THashList labelToName; // liste intermediaire pour accelerer l'acces aux parametres
    UShort_t magic_data;   // Magic word for FASTER data (0xAA) 

  private:
    void GenerateTRefParameter(TString namePar);
    void GenerateTRefTdcParameters(TString namePar);
    void GenerateOscilloParameters(TString namePar);
    void GenerateAdcParameters(TString namePar);
    void GenerateAdcCountParameters(TString namePar);
    void GenerateQdcCountParameters(TString namePar);
    void GenerateQdcTdcParameters(UShort_t qdcType, TString namePar);
    void GenerateQdcTofParameters(UShort_t qdcTofType, TString namePar);
    void GenerateRFParameters(TString namePar);
    void GenerateRFCounterParameters(TString namePar);
    void ReadOscilloData(void);
    void ReadOscilloMetaData(void);
    void ReadQDC(UShort_t nbQ);
    void ReadQDCCount(void);
    void ReadADC(void);
    void ReadADCCount(void);
    void ReadQDC_TDC(UShort_t nbQ);
    void ReadQDC_TOF(UShort_t nbQ);
    void ReadRF(void);
    void ReadRFCounters(void);
    void ReadTimeRef(void);
    void ReadTimeRefTdc(void);
    void ReadGroup(void);

  protected:
    virtual Int_t GetNextEvent(void)=0;
    void GetHeaderDataBytes(void);
    void GetData(void);

  public:
    FFasterDataReader(void);    // constructeur par defaut
    FFasterDataReader(const FFasterDataReader &ap); // constructeur par copie

    ~FFasterDataReader(void);		// desctructeur;

#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
    virtual void Copy(TObject & obj) const;
#else
    virtual void Copy(TObject & obj);
#endif
    FFasterDataReader& operator = (const FFasterDataReader &hp); // operateur =

    virtual void InitFromEnv(TEnv *env);
    virtual void SetDevice(FDevice *dev=0);
    virtual void InitListOfParamsBruts(void);
    virtual void SetParamBrutsValues(void);
    virtual FParamBrut *GetParamBrut(Int_t labelRHB);
    virtual void SetParamDescFile(const Char_t *ipdf) { fileParamDesc = ipdf;}
    virtual const Char_t *GetParamDescFile(void) { return fileParamDesc.Data();}
    virtual const Char_t *GetDataReaderInfo(void) { return (const Char_t *)"";}
    virtual void ResetDataReaderInfo(void) {;}
    
    virtual void AddParam(FParamBrut *p=0); // Ajout d'un parametre aux differentes listes

    ClassDef(FFasterDataReader,1)

};

#endif

